package GrapherGUI;


import java.awt.*;
import java.awt.event.*;
import java.lang.*;
import java.util.*;
import javax.swing.event.*;
import javax.swing.*;


/** Object containing GUI components.
 * @author Bo Shi
 */
public class GUI extends JPanel {

    /** Graph object associated with this Panel.  Not user specified.
     * @see Graph
     */    
    Graph _graph;

    /** User specified parameters for the functions of this graph.
     */        
    SliderMatrix _sm;

    /** The threat managing the animation
     */
    AnimationManager _am;
        
    /** Displays current mouse coordinates on the graph.
     */        
    JLabel _mouseXY;

    /** Show the x-axis zoom toolbar.
     */    
    boolean _show_zoom_x = true;

    /** Show the y-axis zoom toolbar.
     */        
    boolean _show_zoom_y = true;

    /** Show the density toolbar.
     */        
    boolean _show_density = true;

    /** Show the parameters (sliderbar, etc).
     */        
    boolean _show_parameters = true;

    /** Show the limits panel.
     */        
    boolean _show_limits = true;

    /** Constructor.
     * @param sm the SliderMatrix associated with this program.
     * @param minx starting miniumum value of x on the graph plot
     * @param maxx starting maxiumum value of x on the graph plot
     * @param miny starting miniumum value of y on the graph plot
     * @param maxy starting maxiumum value of y on the graph plot
     * @see SliderMatrix
     */        
    public GUI(SliderMatrix sm, AnimationManager am, double minx, double maxx, double miny, double maxy) {
        JPanel main = new JPanel();

        /* The following font bit is taken from
         * http://forum.java.sun.com/thread.jsp?thread=125315&forum=57&message=330309
         * Thanks to 'urmasoft' for the post
         */
        Hashtable oUIDefault = UIManager.getDefaults();
        Enumeration oKey = oUIDefault.keys();
        String oStringKey = null;

        while (oKey.hasMoreElements()) {
            oStringKey = oKey.nextElement().toString();
            if (oStringKey.endsWith("font")
                    || oStringKey.endsWith("acceleratorFont")) {
                UIManager.put(oStringKey, new Font("Dialog", Font.BOLD, 10));
            }
        }
        // end set default font

        main.setLayout(new BoxLayout(main, BoxLayout.Y_AXIS));
        _graph = new Graph(sm, minx, maxx, miny, maxy);
        _sm = sm;
        _am = am;
        SliderListener slistener = new SliderListener();
        JSlider[] slist = _sm.getJSliderArray();

        // create the scroll panel
        JPanel scroll = new JPanel();

        scroll.setLayout(new BoxLayout(scroll, BoxLayout.Y_AXIS));
        if (_am != null) {
            scroll.add(animationPanel());
        }
        for (int i = 0; i < slist.length; i++) {
            scroll.add(_sm.getAllSliders()[i]);
        }
        if (_show_limits == true) {
            scroll.add(limitsPanel());
        }
        JScrollPane scrollpane = new JScrollPane(scroll);

        // add everything to main
        _mouseXY = _graph.mouseLabel();  // link the label to the one controlled by the graph
        main.add(_graph);
        main.add(toolPanel());
        main.add(zoomPanel());
        if (_show_parameters == true) {
            main.add(scrollpane);
        }

        // graph object will listen for changes in the parameter sliders
        for (int i = 0; i < sm.size(); i++) {
            slist[i].addChangeListener(slistener);
        }

        // scrollpane.setPreferredSize(new Dimension(520, 600));
        scrollpane.setPreferredSize(new Dimension(_graph.X, 100));
        add(main);
        setPreferredSize(new Dimension(_graph.X, 730));
    }

    /** Constructor that allows customization of the GUI components shown.
     * @param sm the SliderMatrix associated with this program.
     * @param minx starting miniumum value of x on the graph plot
     * @param maxx starting maxiumum value of x on the graph plot
     * @param miny starting miniumum value of y on the graph plot
     * @param maxy starting maxiumum value of y on the graph plot
     * @param showxzoom set true to display x-axis zoom panel
     * @param showyzoom set true to display y-axis zoom panel
     * @param showdensity set true to display density panel
     * @param showparams set true to display parameter panels
     * @param showlimits set true to display limits panel
     * @see SliderMatrix
     */    
    public GUI(SliderMatrix sm, AnimationManager am, double minx, double maxx, double miny, double maxy,
            boolean showxzoom,
            boolean showyzoom,
            boolean showdensity,
            boolean showparams,
            boolean showlimits) {

        /* The following font bit is taken from
         * http://forum.java.sun.com/thread.jsp?thread=125315&forum=57&message=330309
         * Thanks to 'urmasoft' for the post
         */
        Hashtable oUIDefault = UIManager.getDefaults();
        Enumeration oKey = oUIDefault.keys();
        String oStringKey = null;

        while (oKey.hasMoreElements()) {
            oStringKey = oKey.nextElement().toString();
            if (oStringKey.endsWith("font")
                    || oStringKey.endsWith("acceleratorFont")) {
                UIManager.put(oStringKey, new Font("Dialog", Font.BOLD, 10));
            }
        }
        // end set default font

        _show_zoom_x = showxzoom;
        _show_zoom_y = showyzoom;
        _show_density = showdensity;
        _show_parameters = showparams;
        _show_limits = showlimits;

        _am = am;
		
        JPanel main = new JPanel();

        main.setLayout(new BoxLayout(main, BoxLayout.Y_AXIS));
        _graph = new Graph(sm, minx, maxx, miny, maxy);
        _sm = sm;
        SliderListener slistener = new SliderListener();
        JSlider[] slist = _sm.getJSliderArray();

        // create the scroll panel
        JPanel scroll = new JPanel();

        scroll.setLayout(new BoxLayout(scroll, BoxLayout.Y_AXIS));
        if (_am != null) {
            scroll.add(animationPanel());
        }
        for (int i = 0; i < slist.length; i++) {
            scroll.add(_sm.getAllSliders()[i]);
        }
        JScrollPane scrollpane = new JScrollPane(scroll);

        _mouseXY = _graph.mouseLabel();  // link the label to the one controlled by the graph
        main.add(_graph);
        main.add(toolPanel());
        main.add(zoomPanel());
        main.add(scrollpane);
        main.add(limitsPanel());
        for (int i = 0; i < sm.size(); i++) {
            slist[i].addChangeListener(slistener);
        }
        // scrollpane.setPreferredSize(new Dimension(520, 600));
        scrollpane.setPreferredSize(new Dimension(500, 200));
        add(main);
        setPreferredSize(new Dimension(500, 700));

    }

    /** Returns the <b>Graph</b> object associated with this GUI.
     * @return a <b>Graph</b> object.
     */
    public Graph getGraphObject() {
        return _graph;
    }

    /** Initializes the timer thread that manages the animation and
     * also handles the panel which is created.
     */
    public JPanel animationPanel() {
        JPanel p = new JPanel();
        JPanel tmp;

        // p.setBorder(BorderFactory.createTitledBorder("Animation Controls"));
        p.setBorder(BorderFactory.createLineBorder(Color.black));
        p.setLayout(new GridLayout(2, 4));
        double min, max;

        try {
            String vn = _am.timerVariable();

            min = _sm.min(vn);
            max = _sm.max(vn);
        } catch (Error e) {
            // System.out.println("ERROR: timer variable could not be found");
            return p;
        }

        JButton start = new JButton("Start");
        JButton stop = new JButton("Stop");
        JButton reset = new JButton("Reset");
        JButton setinc = new JButton("Apply");
        final TextNumberInput delta = new TextNumberInput(0.0,
                _sm.max(_am.timerVariable()) / 4.0,
                _sm.max(_am.timerVariable()) / 8.0, 5);
        final TextNumberInput tmin = new TextNumberInput(0.0, 0.0, min, 5);
        final TextNumberInput tmax = new TextNumberInput(0.0, 0.0, max, 5);

        start.setToolTipText("Start the animation.");
        stop.setToolTipText("Stop the animation.");
        reset.setToolTipText("Set the time to its minimum.");
        delta.setToolTipText("Set the time increment between updates");

        start.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                _am.start();
            }
        });
        stop.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                _am.stop();
            }
        });
        reset.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                _am.reset();
            }
        });
        setinc.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                _am.setDeltaT(delta.val());
                ParameterPanel pp = _sm.get(_am.timerVariable());

                if (pp == null) {
                    return;
                }
                pp.resize(tmin.val(), tmax.val());
            }
        });

        tmp = new JPanel();
        tmp.add(new JLabel("t min:"));
        tmp.add(tmin);
        p.add(tmp);
        tmp = new JPanel();
        tmp.add(new JLabel("  t max:"));
        tmp.add(tmax);
        p.add(tmp);
        tmp = new JPanel();
        tmp.add(new JLabel("  dt"));
        tmp.add(delta);
        p.add(tmp);
        p.add(setinc);
        p.add(start);
        p.add(stop);
        p.add(new JLabel("    "));
        p.add(reset);
        return p;
    }
	
    /** Creates 2 checkboxes for showing the key and style of the drawing.
     * @return JPanel with the 2 checkboxes.
     * @see JPanel
     * @see JCheckBox
     */        
    private JPanel toolPanel() {
        JCheckBox dots = new JCheckBox("Only display points", false);
        JCheckBox key = new JCheckBox("Show Legend", true);

        dots.setToolTipText("Uncheck this if you want a smooth graph");
        key.setToolTipText("Get additional information about the plots");
        JPanel p = new JPanel();

        p.setLayout(new GridLayout(1, 3));
        p.add(_mouseXY);
        p.add(dots);
        p.add(key);
        dots.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                _graph.changeStateLinkPoints();
            }
        });
        key.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                _graph.changeStateShowKey();
            }
        });
        return p;
            
    }

    /** Creates a panel with 4 text fields for the upper and lower bounds for the two
     * axes.  Also contains the event handlers.
     * @return JPanel with the correct GUI component.
     */
    private JPanel limitsPanel() {
        JPanel p = new JPanel();
        JPanel limits, ticks;
        final TextNumberInput xmin = new TextNumberInput(0, 0, _graph.minx(), 4);
        final TextNumberInput xmax = new TextNumberInput(0, 0, _graph.maxx(), 4);
        final TextNumberInput ymin = new TextNumberInput(0, 0, _graph.miny(), 4);
        final TextNumberInput ymax = new TextNumberInput(0, 0, _graph.maxy(), 4);
        JButton redraw = new JButton("Apply Changes");

        final TextNumberInput xticks = new TextNumberInput(0, 1,
                _graph.getTickDensityX(), 2);
        final TextNumberInput yticks = new TextNumberInput(0, 1,
                _graph.getTickDensityY(), 2);
        JCheckBox showlabels = new JCheckBox("Show Axis Labels", true);

        redraw.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                // System.out.println("Setting tick density to " + xticks.val() + ", " + yticks.val());
                _graph.setTickDensityX((int) xticks.val());
                _graph.setTickDensityY((int) yticks.val());
                _graph.setDomain(xmin.val(), xmax.val());
                _graph.setRange(ymin.val(), ymax.val());
                _graph.updateTickIncrement();
                _graph.repaint();
            }
        });

        showlabels.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                _graph.changeStateShowLabels();
            }
        });

        limits = new JPanel();
        limits.add(new JLabel("Min x"));
        limits.add(xmin);
        limits.add(new JLabel("  Max x"));
        limits.add(xmax);
        limits.add(new JLabel("  Min y"));
        limits.add(ymin);
        limits.add(new JLabel("  Max y"));
        limits.add(ymax);
        limits.add(redraw);

        ticks = new JPanel();
        ticks.add(new JLabel("x ticks"));
        ticks.add(xticks);
        ticks.add(new JLabel("       y ticks"));
        ticks.add(yticks);
        ticks.add(new JLabel("       "));
        ticks.add(showlabels);

        p.setBorder(BorderFactory.createLineBorder(Color.black));
        p.setLayout(new GridLayout(2, 1));
        p.add(limits);
        p.add(ticks);
        return p;
    }

    /** Creates another panel containing buttons for zoom and setting the density of
     * the graph.
     * @return JPanel containing GUI components
     * @see JPanel
     * @see JButton
     */        
    private JPanel zoomPanel() {
        JButton xplus = new JButton("+");
        JButton xminus = new JButton("-");
        JButton yplus = new JButton("+");
        JButton yminus = new JButton("-");
        JButton redraw = new JButton("Redraw");
        final JTextField density = new JTextField("100", 3);

        xplus.setToolTipText("Zoom out along the x-axis");
        xminus.setToolTipText("Zoom in along the x-axis");
        yplus.setToolTipText("Zoom out along the y-axis");
        yminus.setToolTipText("Zoom in along the y-axis");
        density.setToolTipText("This sets the number of x-y values used to construct the graph (aximum is "
                + _graph.X + ")");
        redraw.setToolTipText("Redraw the graph with the current density setting");

        JPanel p = new JPanel();
        JPanel zx = new JPanel();
        JPanel zy = new JPanel();
        JPanel pd = new JPanel();

        pd.add(density);
        pd.add(redraw);
        pd.setBorder(BorderFactory.createTitledBorder("Density"));
        zx.add(xminus);
        zx.add(xplus);
        zx.setBorder(BorderFactory.createTitledBorder("X-Axis Zoom"));
        zy.add(yminus);
        zy.add(yplus);
        zy.setBorder(BorderFactory.createTitledBorder("Y-Axis Zoom"));

        xplus.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                _graph.zoom(_graph.ZOOM_IN_X);
            }
        });
        xminus.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                _graph.zoom(_graph.ZOOM_OUT_X);
            }
        });
        yplus.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                _graph.zoom(_graph.ZOOM_IN_Y);
            }
        });
        yminus.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                _graph.zoom(_graph.ZOOM_OUT_Y);
            }
        });
        redraw.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                int olddensity = _graph.getDensity();
                int newd;

                try {
                    newd = Integer.parseInt(density.getText());
                } catch (NumberFormatException n) {
                    density.setText("" + olddensity);
                    return;
                }
                if (newd < 0 || newd > _graph.X) {
                    density.setText("" + olddensity);
                    return;
                }
                _graph.setDensity(newd);
                _graph.repaint();
            }
        });

        int i = 0;

        if (_show_density == true) {
            p.add(pd);
            i++;
        }
        ;
        if (_show_zoom_x == true) {
            p.add(zx);
            i++;
        }
        ;
        if (_show_zoom_y == true) {
            p.add(zy);
            i++;
        }
        ;

        p.setLayout(new GridLayout(1, i));
        p.setPreferredSize(new Dimension(500, 60));
            
        return p;
    }
        
    /** Add a function into the graph plot.
     * @param f function object to be added
     * @return true if addition of successful.  False if failure.
     * @see Function
     */        
    public boolean addFunction(Function f) {
        return _graph.addFunc(f);
    }
        
    /** Set the icons that each axis will be labeled
     * @param x file name (relative path) to the image file for the independent axis
     * @param y file name (relative path) to the image file for the dependent axis
     */        
    public void setAxesIcons(String x, String y) {
        _graph.setAxesIcons(x, y);
    }
        
    /** ChangeListener object, event handler for the parameter sliders.
     * @see ChangeListener
     * @see SliderMatrix
     */        
    class SliderListener implements ChangeListener {

        /** Behavior: the graph redraws every time there is a change in slider positions
         * as reported by the JSlider objects contained in SliderMatrix.
         * @see SliderMatrix
         * @see JSlider
         * @param e not used in this function
         */            
        public void stateChanged(ChangeEvent e) {
            _graph.repaint();
        }
    }

    public void setTickIncrementX(double d) {
        _graph.setTickIncrementX(d);
    }

    public void setTickIncrementY(double d) {
        _graph.setTickIncrementY(d);
    }

}
