package GrapherGUI;


import java.util.*;


public class AnimationManager {
    SliderMatrix _smptr;
    String _tv;
    Timer _thread;
    double _dt;				// time interval (microseconds)
    int _refreshRate;

    class MyTask extends TimerTask {
        public void run() {
            if (_smptr.incr(_tv, _dt) == false) {
                stop();
                return;
            }
            _thread.schedule(new MyTask(), _refreshRate);
        }
    }

    /** Constructor
     * @param timevar name of the time parameter -- usually t
     * @param animSpeed the replay speed (milliseconds)
     * @param dt change in time interval per change
     * @param sm parameter data
     */
    public AnimationManager(String timevar, int refreshRate, double dt, SliderMatrix sm) {
        _tv = timevar;
        _smptr = sm;
        _dt = dt;
        _refreshRate = refreshRate;
    }

    public void setDeltaT(double dt) {
        _dt = dt;
    }

    public void setRefresh(int t) {
        _refreshRate = t;
    }

    public void start() {
        if (_thread != null) {
            return;
        }
        // System.out.println("Timer started, incrementing every " + _refreshRate
        // + " milliseconds");
        _thread = new Timer();
        _thread.schedule(new MyTask(), _refreshRate);
    }

    public void reset() {
        if (_thread == null) {
            _smptr.snapLeft(_tv);
            return;
        }
        stop();
        _smptr.snapLeft(_tv);
    }

    public void stop() {
        if (_thread == null) {
            return;
        }
        // System.out.println("Timer stopped!");
        _thread.cancel();
        _thread = null;
    }

    public String timerVariable() {
        return _tv;
    }

}
